#include <fggl/gfx/input.hpp>
#include <cassert>

#include <bitset>
#include <iostream>

using fggl::gfx::Input;

Input::Input() : m_mouse_curr(), m_mouse_last(), m_joydata(), m_joysticks() {
	clear();
}

void Input::clear() {
	m_mouse_curr.cursor[0] = 0.0;
	m_mouse_curr.cursor[1] = 0.0;
	m_mouse_curr.scroll[0] = 0.0;
	m_mouse_curr.scroll[1] = 0.0;
	m_mouse_curr.buttons = 0x00;
	m_mouse_last = m_mouse_curr;
}

void Input::frame() {
	m_mouse_last = m_mouse_curr;
	m_mouse_curr.scroll[0] = 0.0;
	m_mouse_curr.scroll[1] = 0.0;
}

void Input::mousePos(double x, double y) {
	assert( m_mouse_curr.cursor != nullptr );
	m_mouse_curr.cursor[0] = x;
	m_mouse_curr.cursor[1] = y;
}

double Input::cursorDeltaX() const {
	return m_mouse_last.cursor[0] - m_mouse_curr.cursor[0];
}

double Input::cursorDeltaY() const {
	return m_mouse_last.cursor[1] - m_mouse_curr.cursor[1];
}

const double* Input::mousePos() const {
	return m_mouse_curr.scroll.data();
}

void Input::mouseScroll(double deltaX, double deltaY) {
	m_mouse_curr.scroll[0] = deltaX;
	m_mouse_curr.scroll[1] = deltaY;
}

const double* Input::mouseScroll() const {
	return m_mouse_curr.scroll.data();
}

double Input::scrollDeltaX() const {
	return m_mouse_curr.scroll[0];
}

double Input::scrollDeltaY() const {
	return m_mouse_curr.scroll[1];
}

void Input::mouseBtn(const MouseButton btn, bool state) {
	if ( state ) {
		m_mouse_curr.buttons |= btn;
	} else {
		m_mouse_curr.buttons &= ~btn;
	}
}

bool Input::mouseDown(const MouseButton btn) const {
	return m_mouse_curr.buttons & btn;
}

bool Input::mousePressed(const MouseButton btn) const {
	return (m_mouse_curr.buttons & btn) && !(m_mouse_last.buttons & btn);
}

bool Input::mouseReleased(const MouseButton btn) const {
	return !(m_mouse_curr.buttons & btn) && (m_mouse_last.buttons & btn);
}

void Input::joystickConnect(int id, Joystick &data){
	std::cout << "JOYSTICK TIME: " << data.name << std::endl;
	m_joysticks[id] = true;
	m_joydata[id] = data;
}

void Input::joystickDisconnect( int id ){
	// reset to empty joystick
	m_joysticks[id] = false;
	m_joydata[id] = Joystick();
}

bool Input::joystick(int id) const {
	return m_joysticks[id];
}
