#ifndef FGGL_GFX_INPUT_H
#define FGGL_GFX_INPUT_H

#include <array>
#include <bitset>

namespace fggl::gfx {

	enum MouseButton {
		MOUSE_1 = 0b00000001,
		MOUSE_2 = 0b00000010,
		MOUSE_3 = 0b00000100,
		MOUSE_4 = 0b00001000,
		MOUSE_5 = 0b00010000,
		MOUSE_6 = 0b00100000,
		MOUSE_7 = 0b01000000,
		MOUSE_8 = 0b10000000,
	};

	struct MouseInputState {
		std::array<double, 2> cursor;
		std::array<double, 2> scroll;
		char buttons;
	};

	struct Joystick {
		const char* name = nullptr;
		const float* axes = nullptr;
		const unsigned char* buttons = nullptr;
		const unsigned char* hats = nullptr;
		int hatCount = 0;
		int axisCount = 0;
		int buttonCount = 0;
	};

	class Input {
		public:
			// this is a neccerry evil due to glfw's design :'(
			static Input& instance() {
				static Input *instance = new Input();
				return *instance;
			}

			void clear();
			void frame();

			// state motification
			void mousePos(double x, double y);
			void mouseScroll(double deltaX, double deltaY);
			void mouseBtn(const MouseButton btn, bool state);

			// mouse position
			const double* mousePos() const;
			double cursorDeltaX() const;
			double cursorDeltaY() const;

			// mouse scroll
			const double* mouseScroll() const;
			double scrollDeltaX() const;
			double scrollDeltaY() const;

			/**
			 * is the mouse button down this frame?
			 *
			 * True is the mouse button is down
			 */
			bool mouseDown(const MouseButton btn) const;

			/**
			 * Was the mouse button pressed this frame?
			 *
			 * True if the mouse button is down, but was up last frame
			 */
			bool mousePressed(const MouseButton btn) const;

			/**
			 * Was the mouse button released this frame?
			 *
			 * True if the mouse button is up, but was down last frame
			 */
			bool mouseReleased(const MouseButton btn) const;

			// joysticks
			void joystickConnect(int id, Joystick& data);
			void joystickDisconnect(int id);

			bool joystick(int id) const;

		private:
			Input();
			MouseInputState m_mouse_curr;
			MouseInputState m_mouse_last;
			std::bitset<16> m_joysticks;
			std::array<Joystick, 16> m_joydata;
	};

};

#endif
